// Description: 
// By Sarah Dean
// Email: sdean12@softhome.net
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


#include "FreeOTFECypherImpl.h"
#include "FreeOTFECypherNull.h"

#include "FreeOTFEDebug.h"
#include "FreeOTFElib.h"


// =========================================================================
// Cypher driver init function
// devExt - The device extension to be initialized
NTSTATUS
ImpCypherDriverExtDetailsInit(
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;
    int idx = -1;

    DEBUGOUTCYPHERIMPL(DEBUGLEV_ENTER, ("ImpCypherDriverExtDetailsInit\n"));


    // -- POPULATE DRIVER IDENTIFICATION --
    RtlZeroMemory(devExt->DriverTitle, sizeof(devExt->DriverTitle));
    RtlCopyMemory(
                  devExt->DriverTitle,
                  DRIVER_TITLE,
                  strlen(DRIVER_TITLE)
                 );

    devExt->DriverGUID = DRIVER_GUID;
    devExt->DriverVersionID = DRIVER_CYPHER_VERSION;


    // -- POPULATE cyphers SUPPORTED --
    devExt->CypherDetails = ExAllocatePool(
                                         NonPagedPool,
                                         (sizeof(CYPHER) * CYPHERS_SUPPORTED)
                                        );    

    devExt->CypherCount = CYPHERS_SUPPORTED;



    // -- Null --
    idx++;
    RtlZeroMemory(devExt->CypherDetails[idx].Title, MAX_CYPHER_TITLE);
    RtlCopyMemory(
                  devExt->CypherDetails[idx].Title,
                  DRIVER_CIPHER_TITLE_NULL,
                  strlen(DRIVER_CIPHER_TITLE_NULL)
                 );

    devExt->CypherDetails[idx].Mode       = CYPHER_MODE_NONE;
    devExt->CypherDetails[idx].KeySize    = 0;
    devExt->CypherDetails[idx].BlockSize  = -1;
    devExt->CypherDetails[idx].VersionID  = DRIVER_CYPHER_IMPL_VERSION;
    devExt->CypherDetails[idx].CypherGUID = CIPHER_GUID_NULL;



    DEBUGOUTCYPHERIMPL(DEBUGLEV_EXIT, ("ImpCypherDriverExtDetailsInit\n"));

    return status;
}


// =========================================================================
// Cypher driver cleardown function
// devExt - The device extension to be cleared down
NTSTATUS
ImpCypherDriverExtDetailsCleardown(    
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;

    DEBUGOUTCYPHERIMPL(DEBUGLEV_ENTER, ("ImpCypherDriverExtDetailsCleardown\n"));

    if (devExt->CypherDetails != NULL)
        {
        ExFreePool(devExt->CypherDetails);
        }

    devExt->CypherDetails = NULL;
    devExt->CypherCount = 0;

    DEBUGOUTCYPHERIMPL(DEBUGLEV_EXIT, ("ImpCypherDriverExtDetailsCleardown\n"));

    return status;
}


// =========================================================================
// Encryption function
// Note: CyphertextLength must be set to the size of the CyphertextData buffer on
//       entry; on exit, this will be set to the size of the buffer used.
NTSTATUS
ImpCypherEncryptData(
    IN      GUID* CypherGUID,
    IN      int KeyLength,  // In bits
    IN      char *Key,
    IN      char *KeyASCII,  // ASCII representation of "Key"
    IN      int IVLength,  // In bits
    IN      char *IV,
    IN      int PlaintextLength,  // In bytes
    IN      char *PlaintextData,
    OUT     char *CyphertextData
)
{
    NTSTATUS status = STATUS_SUCCESS;

    DEBUGOUTCYPHERIMPL(DEBUGLEV_ENTER, ("ImpCypherEncryptData\n"));

    if (!(IsEqualGUID(&CIPHER_GUID_NULL, CypherGUID)))
        {
        status = STATUS_INVALID_PARAMETER;
        }
    else
        {
        if ( (IV == NULL) && (IVLength != 0) )
            {
            // Sanity check failed...
            DEBUGOUTCYPHERIMPL(DEBUGLEV_INFO, ("Inconsistant: NULL passed as %d bit long IV\n", IVLength));
            status = STATUS_INVALID_PARAMETER;
            }
        else
            {
            RtlCopyMemory(            
                          CyphertextData,
                          PlaintextData,
                          PlaintextLength
                         );    
            }
        }

    DEBUGOUTCYPHERIMPL(DEBUGLEV_EXIT, ("ImpCypherEncryptData\n"));

    return status;
}


// =========================================================================
// Decryption function
// Note: PlaintextLength must be set to the size of the PlaintextData buffer on
//       entry; on exit, this will be set to the size of the buffer used.
NTSTATUS
ImpCypherDecryptData(
    IN      GUID* CypherGUID,
    IN      int KeyLength,  // In bits
    IN      char *Key,
    IN      char *KeyASCII,  // ASCII representation of "Key"
    IN      int IVLength,  // In bits
    IN      char *IV,
    IN      int CyphertextLength,  // In bytes
    IN      char *CyphertextData,
    OUT     char *PlaintextData
)
{
    NTSTATUS status = STATUS_SUCCESS;

    DEBUGOUTCYPHERIMPL(DEBUGLEV_ENTER, ("ImpCypherDecryptData\n"));

    if (!(IsEqualGUID(&CIPHER_GUID_NULL, CypherGUID)))
        {
        status = STATUS_INVALID_PARAMETER;
        }
    else
        {
        if ( (IV == NULL) && (IVLength != 0) )
            {
            // Sanity check failed...
            DEBUGOUTCYPHERIMPL(DEBUGLEV_INFO, ("Inconsistant: NULL passed as %d bit long IV\n", IVLength));
            status = STATUS_INVALID_PARAMETER;
            }
        else
            {
            RtlCopyMemory(            
                          PlaintextData,
                          CyphertextData,
                          CyphertextLength
                         );
            }
        }

    DEBUGOUTCYPHERIMPL(DEBUGLEV_EXIT, ("ImpCypherDecryptData\n"));

    return status;
}


// =========================================================================
// =========================================================================

